--------------------------------------------------------------------------------
--
-- Title       : Uart
-- Design      : Uart Receive
-- Author      : Alex Shtengel
-- Company     : Home
--------------------------------------------------------------------------------
-- File        : UartReceive.vhd
-- Generated   : Feb 10  2009
--------------------------------------------------------------------------------
-- Description : Receiver
--------------------------------------------------------------------------------
-- Revision History :
-- -----------------------------------------------------------------------------
--   Ver  :| Author            :| Mod. Date :|    Changes Made:
--   v1.0  | Alex Shtengel	   :| 10/02/09	:| Automatically Generated
-- -----------------------------------------------------------------------------
library ieee ;

use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity UartReceive  is
port
(
 ClockIn ,        												 																			-- General Clock
 nResetIn ,         											 																			-- General Reset active High
 BasicP16ClockIn,                        	 																			-- Basic  Clock x 16
 SerialDataIn,            								 																			-- Serial data from line
 nReadIn             											 																			-- Read Received Data active 0
                    : in std_logic;
 nRxReadyOut             									 																			-- Received Data Ready active 0
                    : out std_logic;
 ReceiveDataOut             							 																			-- Received Data
                    : out std_logic_vector(7 downto 0)
);
end UartReceive;

architecture  ArchUartReceive of UartReceive is
type ReceiveState  is (Idle,WaitStopBit,WaitStopBitTime,WaitStartBit,WaitStartBitTime,
											 LatchData,WaitBitTime,EndTransaction);
signal ReceiveCurrentState,ReceiveNextState
										: ReceiveState := Idle;

signal nRead,             									-- Read Received Data active 0
			 SerialData            								-- Serial data from line
										: std_logic  := '1';
signal ReceiveData                         	-- Received data
										: std_logic_vector(7 downto 0) := (others => '0');
signal BitsCount														-- Count bits in word
										: integer range 0 to 7 := 7;
signal BitTimeCount													-- Count 16 clocks in bit time
										: integer range 0 to 15 :=15;

begin
ReceiveDataOut <= ReceiveData ;           	-- Received Data

 nRead_Process:
process (ClockIn ,nResetIn)
begin
 if(nResetIn = '0') then
	nRead <= '1';             								-- Read Received Data active 0
	SerialData <= '1';            						-- Serial data from line
 elsif (ClockIn'event and ClockIn = '1') then
	nRead 			<= nReadIn;
	SerialData 	<= SerialDataIn;            		-- Serial data from line
 end if;
end process  nRead_Process;

 ReceiveCurrentState_Process:
process (ClockIn ,nResetIn)
begin
 if(nResetIn = '0') then
	ReceiveCurrentState <= Idle;
 elsif (ClockIn'event and ClockIn = '1') then
	ReceiveCurrentState <= ReceiveNextState;
 end if;
end process  ReceiveCurrentState_Process;

 ReceiveNextState_Process:
process (ClockIn)
begin
 if (ClockIn'event and ClockIn = '0') then
	case ReceiveCurrentState is
	 when Idle =>
		ReceiveNextState <= WaitStopBit;
	 when WaitStopBit =>
		case SerialData is
		 when '1' =>
		  ReceiveNextState <= WaitStopBitTime;
		 when others =>
		  ReceiveNextState <= WaitStopBit;
		end case;
	 when WaitStopBitTime =>
		case SerialData is
		 when '1' =>
			case BitTimeCount is
		 	 when 0 =>
				case BasicP16ClockIn is
				 when '1' =>
				  ReceiveNextState <= WaitStartBit;
				 when others =>
				  ReceiveNextState <= WaitStopBitTime;
				end case;
		 	 when others =>
		  	ReceiveNextState <= WaitStopBitTime;
			end case;
		 when others =>
			case BitTimeCount is
		 	 when 0 =>
		  	ReceiveNextState <= WaitStartBit;
		 	 when others =>
				ReceiveNextState <= WaitStopBit;
			end case;
		end case;
	 when WaitStartBit =>
		case SerialData is
		 when '0' =>
		  ReceiveNextState <= WaitStartBitTime;
		 when others =>
		  ReceiveNextState <= WaitStartBit;
		end case;
	 when WaitStartBitTime =>
		case SerialData is
		 when '0' =>
			case BitTimeCount is
		 	 when 0 =>
				case BasicP16ClockIn is
				 when '1' =>
				   ReceiveNextState <= LatchData;
				 when others =>
				  ReceiveNextState <= WaitStartBitTime;
				end case;
		 	 when others =>
				ReceiveNextState <= WaitStartBitTime;
			end case;
		 when others =>
		  ReceiveNextState <= WaitStopBitTime;
		end case;
	 when LatchData =>
		ReceiveNextState <= WaitBitTime;
	 when WaitBitTime =>
		case BitTimeCount is
		 when 0 =>
			case BitsCount is
			 when 0 =>
				case BasicP16ClockIn is
				 when '1' =>
				  ReceiveNextState <= EndTransaction;
				 when others =>
				  ReceiveNextState <= WaitBitTime;
				end case;
			 when others =>
				case BasicP16ClockIn is
				 when '1' =>
				  ReceiveNextState <= LatchData;
				 when others =>
				  ReceiveNextState <= WaitBitTime;
				end case;
			end case;
		 when others =>
		  ReceiveNextState <= WaitBitTime;
		end case;
	 when EndTransaction =>
		if(nRead = '0') then
		 ReceiveNextState <= WaitStopBit;
		else
		 ReceiveNextState <= EndTransaction;
		end if;
	 when others =>
	  ReceiveNextState <= Idle;
	end case;
 end if;
end process  ReceiveNextState_Process;

 BitTimeCount_Process:
process (ClockIn ,nResetIn)
begin
 if(nResetIn = '0') then
	BitTimeCount <= 15;												-- Count 16 clocks in bit time
 elsif (ClockIn'event and ClockIn = '1') then
	case ReceiveNextState is
	 when Idle =>
		BitTimeCount <= 13;
	 when WaitStopBit =>
		BitTimeCount <= 13;
	 when WaitStopBitTime =>
		case ReceiveCurrentState is
		 when WaitStopBitTime =>
	 		case BasicP16ClockIn is
	 		 when '1' =>
	 		  BitTimeCount <= BitTimeCount -1 mod 15;
	 		 when others =>
	 		  BitTimeCount <= BitTimeCount;
	 		end case;
		 when others =>
		  BitTimeCount <= BitTimeCount;
		end case;
	 when WaitStartBit =>
		BitTimeCount <= 12;
	 when WaitStartBitTime =>
		case ReceiveCurrentState is
		 when WaitStartBitTime =>
	 		case BasicP16ClockIn is
	 		 when '1' =>
	 		  BitTimeCount <= BitTimeCount -1 mod 15;
	 		 when others =>
	 		  BitTimeCount <= BitTimeCount;
	 		end case;
		 when others =>
		  BitTimeCount <= BitTimeCount;
		end case;
	 when LatchData =>
		BitTimeCount <= 15;
	 when WaitBitTime =>
		case BasicP16ClockIn is
		 when '1' =>
		  BitTimeCount <= BitTimeCount -1 mod 15;
		 when others =>
		  BitTimeCount <= BitTimeCount;
		end case;
	 when others =>
	  BitTimeCount <= BitTimeCount;
	end case;
 end if;
end process  BitTimeCount_Process;

 BitsCount_Process:
process (ClockIn ,nResetIn)
begin
 if(nResetIn = '0') then
	BitsCount	<= 7;														-- Count bits in word
 elsif (ClockIn'event and ClockIn = '1') then
	case ReceiveNextState is
	 when Idle =>
	  BitsCount	<= 7;
	 when WaitStartBit =>
	  BitsCount	<= 7;
	 when LatchData =>
		case ReceiveCurrentState is
		 when WaitBitTime =>
		  BitsCount	<= BitsCount - 1 mod 7;
		 when others =>
		  BitsCount	<= BitsCount;
		end case;
	 when others =>
	  BitsCount	<= BitsCount;
	end case;
 end if;
end process  BitsCount_Process;


 nRxReadyOut_Process:
process (ClockIn ,nResetIn)
begin
 if(nResetIn = '0') then
	nRxReadyOut <= '1';            						-- Received Data Ready active 0
 elsif (ClockIn'event and ClockIn = '1') then
	case ReceiveNextState is
	 when EndTransaction =>
	  nRxReadyOut <= '0';
	 when others =>
	  nRxReadyOut <= '1';
	end case;
 end if;
end process  nRxReadyOut_Process;

 ReceiveData_Process:
process (ClockIn ,nResetIn)
begin
 if(nResetIn = '0') then
	ReceiveData <= (others => '0');           -- Received data
 elsif (ClockIn'event and ClockIn = '1') then
	case ReceiveNextState is
	 when WaitStartBit =>
		ReceiveData <= (others => '0');
	 when WaitBitTime =>
		case BitTimeCount is
		 when 7 =>
			case BitsCount is
			 when 0 =>
				ReceiveData(7) <= SerialData;
			 when 1 =>
				ReceiveData(6) <= SerialData;
			 when 2 =>
				ReceiveData(5) <= SerialData;
			 when 3 =>
				ReceiveData(4) <= SerialData;
			 when 4 =>
				ReceiveData(3) <= SerialData;
			 when 5 =>
				ReceiveData(2) <= SerialData;
			 when 6 =>
				ReceiveData(1) <= SerialData;
			 when 7 =>
				ReceiveData(0) <= SerialData;
			 when others =>
				ReceiveData <= ReceiveData;
			end case;
		 when others =>
		  ReceiveData <= ReceiveData;
		end case;
	 when others =>
	  ReceiveData <= ReceiveData;
	end case;
 end if;
end process  ReceiveData_Process;

end ArchUartReceive;
