--------------------------------------------------------------------------------
--
-- Title       : Uart
-- Design      : Uart Transmit
-- Author      : Alex Shtengel
-- Company     : Home
--------------------------------------------------------------------------------
-- File        : UartTransmit.vhd
-- Generated   : Feb 10  2009
--------------------------------------------------------------------------------
-- Description : Transmitter RS232
--------------------------------------------------------------------------------
-- Revision History :
-- -----------------------------------------------------------------------------
--   Ver  :| Author            :| Mod. Date :|    Changes Made:
--   v1.0  | Alex Shtengel	   :| 10/02/09	:| Automatically Generated
-- -----------------------------------------------------------------------------
library ieee ;

use ieee.std_logic_1164.all ;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity UartTransmit  is
port
(
 ClockIn ,        												                                    	-- General Clock
 nResetIn ,        												                                    	-- General Reset active High
 BasicP16ClockIn,                        	                                    	-- Basic  Clock x 16
 nWriteIn             										                                    	-- Write data for transmit
                    : in std_logic;
 TransmitDataIn             							                                    	-- Data for transmit
                    : in std_logic_vector(7 downto 0);
 nTxReadyOut,            																												-- Transmit Ready active 0
 SerialDataOut             																											-- Serial data for Transmit
                    : out std_logic
);
end UartTransmit;

architecture  ArchUartTransmit of UartTransmit is
type TransmitState  is (IdleWrite,WaitBasicClock,StartBit,LatchData,WaitBitTime,
												StopBit,EndTransaction);
signal TransmitCurrentState,TransmitNextState
										: TransmitState := IdleWrite;

signal nWrite              																											-- Write data for transmit
										: std_logic := '1';
signal TransmitData                         																		-- Data for transmit
										: std_logic_vector(7 downto 0) := (others => '0');
signal BitsCount																																-- Count bits in word
										: integer range 0 to 7 := 7;
signal BitTimeCount																															-- Count 16 clocks in bit time
										: integer range 0 to 15 := 15;

begin

 nWrite_Process:
process (ClockIn ,nResetIn,nWriteIn)
begin
 if(nResetIn = '0') then
	nWrite <= '1';             																										-- Write data for transmit
 elsif (ClockIn'event and ClockIn = '1') then
	nWrite <= nWriteIn;
 end if;
end process  nWrite_Process;

 TransmitCurrentState_Process:
process (ClockIn ,nResetIn)
begin
 if(nResetIn = '0') then
	TransmitCurrentState <= IdleWrite;
 elsif (ClockIn'event and ClockIn = '1') then
	TransmitCurrentState <= TransmitNextState;
 end if;
end process  TransmitCurrentState_Process;

 TransmitNextState_Process:
process (ClockIn)
begin
 if (ClockIn'event and ClockIn = '0') then
	case TransmitCurrentState is
	 when IdleWrite =>
		case nWrite is
		 when '0' =>
		  TransmitNextState <= WaitBasicClock;
		 when others =>
		  TransmitNextState <= IdleWrite;
		end case;
	 when WaitBasicClock =>
		case BasicP16ClockIn is
		 when '1' =>
		  TransmitNextState <= StartBit;
		 when others =>
		  TransmitNextState <= WaitBasicClock;
		end case;
	 when StartBit =>
		case BitTimeCount is
		 when 0 =>
			case BasicP16ClockIn is
			 when '1' =>
			  TransmitNextState <= LatchData;
			 when others =>
			  TransmitNextState <= StartBit;
			end case;
		 when others =>
		  TransmitNextState <= StartBit;
		end case;
	 when LatchData =>
		TransmitNextState <= WaitBitTime;
	 when WaitBitTime =>
 		case BitTimeCount is
		 when 0 =>
			case BasicP16ClockIn is
			 when '1' =>
				case BitsCount is
			 	 when 0 =>
			  	TransmitNextState <= StopBit;
			 	 when others =>
			  	TransmitNextState <= LatchData;
				end case;
			 when others =>
				TransmitNextState <= WaitBitTime;
			end case;
		 when others =>
		  TransmitNextState <= WaitBitTime;
		end case;
	 when StopBit =>
		case BitTimeCount is
		 when 0 =>
			case BasicP16ClockIn is
			 when '1' =>
			  TransmitNextState <= EndTransaction;
			 when others =>
			  TransmitNextState <= StopBit;
			end case;
		 when others =>
		  TransmitNextState <= StopBit;
		end case;
	 when EndTransaction =>
		if(nWrite = '1') then
		 TransmitNextState <= IdleWrite;
		else
		 TransmitNextState <= EndTransaction;
		end if;
	 when others =>
	  TransmitNextState <= IdleWrite;
	end case;
 end if;
end process  TransmitNextState_Process;

 BitTimeCount_Process:
process (ClockIn ,nResetIn)
begin
 if(nResetIn = '0') then
	BitTimeCount <= 15;																														-- Count 16 clocks in bit time
 elsif (ClockIn'event and ClockIn = '1') then
	case TransmitNextState is
	 when IdleWrite =>
		BitTimeCount <= 15;
	 when WaitBasicClock =>
		BitTimeCount <= 15;
	 when StartBit =>
		case TransmitCurrentState is
		 when StartBit =>
	 		case BasicP16ClockIn is
	 		 when '1' =>
	 		  BitTimeCount <= BitTimeCount -1 mod 15;
	 		 when others =>
	 		  BitTimeCount <= BitTimeCount;
	 		end case;
		 when others =>
		  BitTimeCount <= BitTimeCount;
		end case;
	 when LatchData =>
		BitTimeCount <= 15;
	 when WaitBitTime =>
		case BasicP16ClockIn is
		 when '1' =>
		  BitTimeCount <= BitTimeCount -1 mod 15;
		 when others =>
		  BitTimeCount <= BitTimeCount;
		end case;
	 when StopBit =>
		case TransmitCurrentState is
		 when WaitBitTime =>
		  BitTimeCount <= 15;
		 when others =>
 		  case BasicP16ClockIn is
		   when '1' =>
		    BitTimeCount <= BitTimeCount -1 mod 15;
		   when others =>
			BitTimeCount <= BitTimeCount;
		 end case;
	  end case;
	 when others =>
	  BitTimeCount <= BitTimeCount;
	end case;
 end if;
end process  BitTimeCount_Process;

 BitsCount_Process:
process (ClockIn ,nResetIn)
begin
 if(nResetIn = '0') then
	BitsCount	<= 7;																																-- Count bits in word
 elsif (ClockIn'event and ClockIn = '1') then
	case TransmitNextState is
	 when IdleWrite =>
	  BitsCount	<= 7;
	 when LatchData =>
		case TransmitCurrentState is
		 when WaitBitTime =>
		  BitsCount	<= BitsCount - 1 mod 7;
		 when others =>
		  BitsCount	<= BitsCount;
		end case;
	 when others =>
	  BitsCount	<= BitsCount;
	end case;
 end if;
end process  BitsCount_Process;

 nTxReadyOut_Process:
process (ClockIn ,nResetIn)
begin
 if(nResetIn = '0') then
	nTxReadyOut <= '1';            																								-- Transmit Ready active 0
 elsif (ClockIn'event and ClockIn = '1') then
	case TransmitNextState is
	 when IdleWrite =>
	  nTxReadyOut <= '0';
	 when others =>
	  nTxReadyOut <= '1';
	end case;
 end if;
end process  nTxReadyOut_Process;

 SerialDataOut_Process:
process (ClockIn ,nResetIn)
begin
 if(nResetIn = '0') then
	SerialDataOut <= '1';            																							-- Serial data for Transmit
	TransmitData  <= (others => '0');         																		-- Data for trnsmit
 elsif (ClockIn'event and ClockIn = '1') then
	case TransmitNextState is
	 when IdleWrite =>
		SerialDataOut <= '1';
		TransmitData  <= TransmitData;
	 when WaitBasicClock =>
		SerialDataOut <= '1';
		case TransmitCurrentState is
		 when IdleWrite =>
		  TransmitData  <= TransmitDataIn;
		 when others =>
		  TransmitData  <= TransmitData;
		end case;
	 when StartBit =>
		SerialDataOut <= '0';
		TransmitData  <= TransmitData;
	 when LatchData =>
		SerialDataOut <= TransmitData(0);
		case TransmitCurrentState is
		 when WaitBitTime =>
		  TransmitData(6 downto 0)  <= TransmitData (7 downto 1);
		 when others =>
		  TransmitData  <= TransmitData;
		end case;
	 when WaitBitTime =>
		SerialDataOut <= TransmitData(0);
		TransmitData  <= TransmitData;
	 when StopBit =>
		SerialDataOut <= '1';
	 when EndTransaction =>
		SerialDataOut <= '1';
	 when others =>
	  SerialDataOut <= '1';
	end case;
 end if;
end process  SerialDataOut_Process;

end ArchUartTransmit;
